; $Id: dcs_docorrections.pro,v 1.10 2002/07/15 21:05:13 john Exp $
; Written by J.R.D. Copley.
;************************************************************************************************
pro dcs_define_qflagPtr,ngroups,elasticqvals,davePtr=davePtr,qflagPtr=qflagPtr
;************************************************************************************************
; This procedure is called if the grouping is in Q.
; First we construct the array Q(t,d), whose pointer is inelasticqPtr.
;	Then we construct a 2-d grouping array whose pointer is tdgPtr
;		('two dimensional group pointer').
;	Then we construct an array of pointers, qflagPtr, to 2-d arrays of 1s and 0s.
; All of these arrays are dimensioned ngtchan0*nhedet.
;
compile_opt strictarr
;
device,decomposed=0
loadct,13
;
progressBar = Obj_New($
	"SHOWPROGRESS",$
;	/cancelbutton,$;	(How does this work?)
	title="Work in progress. Please wait.",$
	xsize=300,$
	ysize=10,$
	message="Please be patient...",$
	color=184)
progressBar->Start
;
nhedet=(*(*(*davePtr).dataStrPtr).specificPtr).nhedet
dcs_getangles,angles,nhedet=nhedet
cos2th=cos((!pi/180.)*angles)
unitvec_d=intarr(nhedet)+1
;
lambda=(*(*(*davePtr).dataStrPtr).specificPtr).ch_wl
e0=float(!dcs_hsq2mn/lambda^2)
;
ngtchan0=(*(*(*davePtr).dataStrPtr).specificPtr).ngtchan0
efinal=e0-float((*(*(*(*davePtr).dataStrPtr).specificPtr).energy_propsptr)[0:ngtchan0-1])
unitvec_t=intarr(ngtchan0)+1
;
conversion=float(4.*!pi^2/!dcs_hsq2mn)
qsqptr=ptr_new(conversion*(e0+efinal#unitvec_d-2.*sqrt(e0*efinal)#cos2th),/no_copy)
sign_of_2th=(angles gt 0)*2-1
if (elasticqvals[0] lt 0.0) then $
	inelasticqPtr=ptr_new(sqrt(*qsqptr)*(unitvec_t#sign_of_2th),/no_copy) else $
	inelasticqPtr=ptr_new(sqrt(*qsqptr),/no_copy)
ptr_free,qsqptr
;
elasticqvals=float(elasticqvals)
tdgPtr=ptr_new(bytarr(ngtchan0,nhedet))
*tdgPtr=*inelasticqPtr ge elasticqvals[0] and *inelasticqPtr lt elasticqvals[1]
;
progressBar->Update,10
;
for k=1,ngroups-1 do begin
	*tdgPtr=*tdgPtr+(k+1)*$
		(*inelasticqPtr ge elasticqvals[k] and *inelasticqPtr lt elasticqvals[k+1])
	progressBar->Update,10+k/float(ngroups-1)*80
endfor
;
ptr_free,inelasticqPtr
;
qflagPtr=ptrarr(ngroups,/allocate_heap,/nozero)
;
progressBar->Update,100
;
for k=0,ngroups-1 do begin
	*qflagPtr[k]=(*tdgPtr eq k)
endfor
;
ptr_free,tdgPtr
;
progressBar->Destroy
Obj_Destroy, progressBar
;
device,decomposed=1
;
end


;************************************************************************************************
pro dcs_redefine_props,angle_props,Q_props,ngroups,davePtr=davePtr
;************************************************************************************************
; This procedure is called if the grouping is in Q.
; It redefines angle_props and Q_props.
; Note that (*Q_propsPtr)[*,0] will become the y component in histPtr.
;
compile_opt strictarr
;
;
lambda=(*(*(*davePtr).dataStrPtr).specificPtr).ch_wl
e0=float(!dcs_hsq2mn/lambda^2)
conversion=float(4.*!pi^2/!dcs_hsq2mn)
k0=sqrt(conversion*e0)
;
; The variable qrat is a 2-column array containing absolute (Q/k_0) limits for each group.
qrat=transpose(abs(Q_props[*,1:2])/k0)
;
; The variables min2th and max2th contain minimum and maximum values of abs(2theta).
nhedet=(*(*(*davePtr).dataStrPtr).specificPtr).nhedet
dcs_getangles,angles,nhedet=nhedet
min2th=min(abs(angles))
max2th=max(abs(angles))
cmin2th=cos(min2th*!dtor)
cmax2th=cos(max2th*!dtor)
argmin=cmin2th^2+qrat^2-1
argmax=cmax2th^2+qrat^2-1
;
; Expand Q_props to 7 rows.
; Rows 1-3 contain the mean, minimum and maximum Q for the group.
; Rows 4-5 contain the minimum and maximum energy transfer for the minimum Q in the group.
; Rows 6-7 contain the minimum and maximum energy transfer for the maximum Q in the group.
Q_props=[[Q_props],[fltarr(ngroups,4)]]
; Compute the quantities that go in rows 4-7 of Q_props.
jk=-1
for j=0,ngroups-1 do begin
	for k=0,1 do begin
		jk=jk+1
		elim=fltarr(2)
		nr=-1
		if (argmin[jk] gt 0.0) then begin
			roomin=sqrt(argmin[jk])
			if (qrat[jk] gt 1.0) then begin
				nr=nr+1
				elim[nr]=e0*(cmin2th+roomin)^2
			endif
			if (qrat[jk] le 1.0 and cmin2th gt 0.0) then begin
				nr=nr+1
				elim[nr]=e0*(cmin2th-roomin)^2
				nr=nr+1
				elim[nr]=e0*(cmin2th+roomin)^2
			endif
		endif
		if (argmax[jk] gt 0.0) then begin
			roomax=sqrt(argmax[jk])
			if (qrat[jk] gt 1.0) then begin
				nr=nr+1
				elim[nr]=e0*(cmax2th+roomax)^2
			endif
			if (qrat[jk] le 1.0 and cmax2th gt 0.0) then begin
				nr=nr+1
				elim[nr]=e0*(cmax2th-roomax)^2
				nr=nr+1
				elim[nr]=e0*(cmax2th+roomax)^2
			endif
		endif
		elim=e0-elim
		elim=elim[sort(elim)]
		Q_props[j,3+k*2]=elim[0]
		Q_props[j,3+k*2+1]=elim[1]
	endfor
endfor
end


;************************************************************************************************
pro dcs_make_eta,det_eff_qty,det_eff_err,grouping,ngroups,doQgrouping,$
	qflagPtr,ngtchan0,nhedet,sumeff_qty,sumeff_err
;************************************************************************************************
; This procedure constructs the quantity eta-hat(t,g).
; Quantities returned are dimensioned ngtchan0*ngroups.
;
compile_opt strictarr
;
unitvec_t=intarr(ngtchan0)+1
;
case ngroups of
;	One group.
	1: begin
		sumeff_qty = unitvec_t*total(det_eff_qty)
		sumeff_err = unitvec_t*sqrt(total(det_eff_err^2 ))
	end
;
; No grouping (hence nhedet groups).
	nhedet: begin
		sumeff_qty = unitvec_t#det_eff_qty
		sumeff_err = unitvec_t#det_eff_err
	end
;
;	Otherwise...
	else: begin
;
; Grouping in 2theta (rather than in Q):
		if (not doQgrouping) then begin
			sumeff_qty=fltarr(ngtchan0,ngroups)
			sumeff_err=fltarr(ngtchan0,ngroups)
			for k=0,ngroups-1 do begin
				list=where(grouping eq k+1)
				sumeff_qty[*,k] = unitvec_t#total(det_eff_qty[list])
				sumeff_err[*,k] = unitvec_t#sqrt(total(det_eff_err[list]^2))
			endfor
		endif else begin

; Grouping in Q (rather than in angle):
			sumeff_qty=fltarr(ngtchan0,ngroups)
			sumeff_err=fltarr(ngtchan0,ngroups)
			for k=0,ngroups-1 do begin
				sumeff_qty[*,k] = total((unitvec_t#det_eff_qty)*(*qflagPtr[k]),2)
				sumeff_err[*,k] = sqrt(total((unitvec_t#det_eff_err^2)*(*qflagPtr[k]),2 ))
			endfor
		endelse
	end
endcase
;
end


;************************************************************************************************
pro dcs_make_ibhat,fbdata_qty,fbdata_err,fbnormalization,grouping,ngroups,doQgrouping,$
	speriod,qflagPtr,ngtchan0,nhedet,sumfb_qty,sumfb_err
;************************************************************************************************
; This procedure constructs the quantity I_B-hat(t,g)/tau.
; Quantities returned are dimensioned ngtchan0*ngroups if tnorm ne 0.
; If tnorm=0 (i.e. T_B^tot = 0), scalar zeros are returned.
;
compile_opt strictarr
;
tnorm=max(fbnormalization)
;
if (tnorm eq 0.0) then begin
	sumfb_qty=0.0
	sumfb_err=0.0
	return
endif
;
unitvec_t=intarr(ngtchan0)+1
;
case ngroups of
;	One group.
	1: begin
		sumfb_qty = unitvec_t*total(fbdata_qty)/speriod
		sumfb_err = unitvec_t*sqrt(total(fbdata_err^2))/speriod
	end
;
; No grouping (hence nhedet groups).
	nhedet: begin
		sumfb_qty = (unitvec_t#fbdata_qty)/speriod
		sumfb_err = (unitvec_t#fbdata_err)/speriod
	end
;
;	Otherwise...
	else: begin
;
; Grouping in 2theta (rather than in Q):
		if (not doQgrouping) then begin
			sumfb_qty=fltarr(ngtchan0,ngroups)
			sumfb_err=fltarr(ngtchan0,ngroups)
			for k=0,ngroups-1 do begin
				list=where(grouping eq k+1)
				sumfb_qty[*,k] = (unitvec_t#(total(fbdata_qty[list])))/speriod
				sumfb_err[*,k] = (unitvec_t#(sqrt(total(fbdata_err[list]^2))))/speriod
			endfor
		endif else begin

; Grouping in Q (rather than in angle):
			sumfb_qty=fltarr(ngtchan0,ngroups)
			sumfb_err=fltarr(ngtchan0,ngroups)
			for k=0,ngroups-1 do begin
				sumfb_qty[*,k] = total((unitvec_t#fbdata_qty)*(*qflagPtr[k]),2)/speriod
				sumfb_err[*,k] = sqrt(total((unitvec_t#fbdata_err^2)*(*qflagPtr[k]),2))/speriod
			endfor
		endelse
	end
endcase
;
end


;************************************************************************************************
pro dcs_docorrections,dcs_base,sumfiles,n_sumfiles,det_eff_qty,det_eff_err,grouping,ngroups,$
	angle_props,Q_props,grpfiles,temperatures,logentry=logentry,elasticqvals=elasticqvals,$
	fbdata_qty=fbdata_qty,fbdata_err=fbdata_err,fbnormalization=fbnormalization,doQgrouping

;************************************************************************************************
; Performs fast background subtraction, detector grouping, correction for detector efficiencies,
; data normalization, and corrections using empty can and cadmium runs.
;
; Input variables:
; sumfiles and n_sumfiles contain information about the .sum files to be used in the procedure.
;	det_eff_qty and det_eff_err are 913-long vectors with obvious contents.
;	grouping and ngroups define, along with doQgrouping, how the data are to be grouped.
;	angle_props and Q_props contain angle and Q information about the groups.
; grpfiles are the names of the grouped data files.
; temperatures
;	elasticqvals
; fbdata_qty and fbdata_err contain background count rates and errors for each detector.
; fbnormalization is a set of 1s and 0s depending whether or not the detector is masked.

compile_opt strictarr
;
res=ptr_valid(count=nptr)
;
; This is the number of files to be treated (S+C+A)
nfiles=fix(n_sumfiles[0]+n_sumfiles[1]+n_sumfiles[3])
;
; Generate the names of the files to be treated.
files=sumfiles[0:n_sumfiles[0]-1]
;
; Extract names of container and absorber files.
if (n_sumfiles[1] gt 0) then begin
	canfile=sumfiles[n_sumfiles[0]]
	files=[files,canfile]
endif else begin
	canfile=""
endelse
;
if (n_sumfiles[3] gt 0) then begin
	absfile=sumfiles[nfiles-1]
	files=[files,absfile]
endif else begin
	absfile=""
endelse

logentry=logentry+string(10B)+$
	"Background subtraction, detector grouping, detector efficiency correction, "+$
	"data normalization and corrections using empty can and cadmium runs have been performed."
;
lines=strarr(2*nfiles+6)
lines[0:2]=[$
	"Performing background subtraction, detector grouping,",$
	"beam monitor normalization, detector efficiency corrections,",$
	"and corrections using empty can and cadmium runs"]
bases=intarr(2*nfiles)
;
grpfiles=strarr(n_sumfiles[0])
temperatures=fltarr(n_sumfiles[0])
;
; Read the first file in order to get at quantities such as ch_wl, nhedet etc.
filename=sumfiles[0]
restore,filename=filename
;
nhedet=(*(*(*davePtr).dataStrPtr).specificPtr).nhedet
ngtchan0=(*(*(*davePtr).dataStrPtr).specificPtr).ngtchan0
;
; Calculate time between pulses at sample.
speriod=6.0e7/(*(*(*davePtr).dataStrPtr).specificPtr).ch_ms*$
	(*(*(*davePtr).dataStrPtr).specificPtr).ch_srdenom
;
if (doQgrouping) then begin
	dcs_redefine_props,angle_props,Q_props,ngroups,davePtr=davePtr
	dcs_define_qflagPtr,ngroups,elasticqvals,davePtr=davePtr,qflagPtr=qflagPtr
endif
;
; The normalization sum is the same for all unmasked detectors and 0 otherwise.
; These two lines reconstruct the detector masking vector M(d).
normalization_sum=(*(*(*davePtr).dataStrPtr).specificPtr).normalization_sum
masking=normalization_sum/max(normalization_sum)
;
; Construct eta-hat(t,g).
dcs_make_eta,det_eff_qty,det_eff_err,grouping,ngroups,doQgrouping,$
	qflagPtr,ngtchan0,nhedet,sumeff_qty,sumeff_err
;
; Construct I_B-hat(t,g)/tau (scalar zeros if T_B^tot=0).
dcs_make_ibhat,fbdata_qty,fbdata_err,fbnormalization,grouping,ngroups,doQgrouping,$
	speriod,qflagPtr,ngtchan0,nhedet,sumfb_qty,sumfb_err
;
f_a=0.0
absfile_qty=0.0
absfile_err=0.0
fa_tnorm=0.0
;
f_c=0.0
canfile_qty=0.0
canfile_err=0.0
fc_tnorm=0.0
;
; Loop over files, decrementing the file index.
for i=nfiles-1,0,-1 do begin
	filename=files[i]
	icomp=nfiles-1-i
	lines[icomp*2+4]="Reading input file "+filename
	dcs_message_display,lines,base,group_leader=dcs_base,alignment=0
	bases[icomp*2]=base
;
	free_all_ptrs,davePtr
	restore,filename=filename
;
;	Put angle_props and Q_props into the DAVE structure.
	free_Ptr,(*(*(*davePtr).dataStrPtr).specificPtr).angle_propsPtr
	free_Ptr,(*(*(*davePtr).dataStrPtr).specificPtr).Q_propsPtr
	(*(*(*davePtr).dataStrPtr).specificPtr).angle_propsPtr=ptr_new(angle_props)
	(*(*(*davePtr).dataStrPtr).specificPtr).Q_propsPtr=ptr_new(Q_props)
;
; The quantities norm and tnorm are N_X^tot and T_X^tot/N_X^tot respectively.
;	We neglect their errors.
	norm=max((*(*(*davePtr).dataStrPtr).specificPtr).normalization_sum)
	tnorm=max((*(*(*davePtr).dataStrPtr).specificPtr).fbnormalization_sum)/norm
;
;	qtyptr points to I_X(t,d), errptr points to its error, xptr points to x.
; Dereference these quantities, free the pointer, redefine the pointer to
; reflect arrays dimensioned ngroups rather than nhedet, populate the structure.
;
	qtyPtr=ptr_new((*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty)
	errPtr=ptr_new((*(*(*davePtr).dataStrPtr).commonStr.histPtr).err)
	xPtr=ptr_new((*(*(*davePtr).dataStrPtr).commonStr.histPtr).x)
;
	free_Ptr,(*(*davePtr).dataStrPtr).commonStr.histPtr
;
	(*(*davePtr).dataStrPtr).commonStr.histPtr=$
		ptr_new({qty:fltarr(ngtchan0,ngroups),$
		err:fltarr(ngtchan0,ngroups),$
		x:*xPtr,$
		y:(*(*(*(*davePtr).dataStrPtr).specificPtr).Q_propsPtr)[*,0]})
;
; In what follows, qty is I_X-hat(t,g)/N_X^tot and err is the
;	corresponding error.
;
	case ngroups of
;	One group.
		1: begin
			(*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty = total((*qtyPtr),2)/norm
			(*(*(*davePtr).dataStrPtr).commonStr.histPtr).err = $
				sqrt(total((*errPtr)^2,2))/norm; this line fixed 5/8/02
		end
;
; No grouping (hence nhedet groups).
		nhedet: begin
			(*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty = *qtyPtr/norm
			(*(*(*davePtr).dataStrPtr).commonStr.histPtr).err = *errPtr/norm
		end
;
;	Otherwise...
		else: begin
;
; Grouping in 2theta (rather than in Q):
			if (not doQgrouping) then begin
				for k=0,ngroups-1 do begin
					list=where(grouping eq k+1)
					(*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty[*,k] = $
						total((*qtyPtr)[*,list],2)/norm
					(*(*(*davePtr).dataStrPtr).commonStr.histPtr).err[*,k] = $
						sqrt(total( (*errPtr)[*,list]^2,2 ))/norm
				endfor
			endif else begin
;
; Grouping in Q (rather than in angle):
				device,decomposed=0
				loadct,13
;
				progressBar = Obj_New($
					"SHOWPROGRESS",$
;					/cancelbutton,$;	(How does this work?)
					title="Work in progress. Please wait.",$
					xsize=300,$
					ysize=10,$
					message="Please be patient...",$
					color=184)
				progressBar->Start
;
				for k=0,ngroups-1 do begin
					(*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty[*,k] = $
						total((*qtyPtr)*(*qflagPtr[k]),2)/norm
					(*(*(*davePtr).dataStrPtr).commonStr.histPtr).err[*,k] = $
						sqrt(total( (*errPtr)^2*(*qflagPtr[k]),2 ))/norm
					progressBar->Update,10+(k+1)/float(ngroups)*90
				endfor
;
				progressBar->Destroy
				Obj_Destroy, progressBar
;
device,decomposed=1
			endelse
		end
	endcase
;
	if (filename eq canfile) then begin
		title="Enter mean sample attenuation factor"
		text="Mean sample attenuation factor"
		value='1.0000'
		dave_get_values,group_leader=dcs_base,title,text,value,/checkall
		f_c=float(value)
		canfile_qty=f_c*(*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty
		canfile_err=f_c*(*(*(*davePtr).dataStrPtr).commonStr.histPtr).err
		fc_tnorm=f_c*tnorm
		lines[icomp*2+4]="Processing container file "+filename
		dcs_message_display,lines,base,group_leader=dcs_base,alignment=0
		bases[icomp*2+1]=base
	endif
;
	if (filename eq absfile) then begin
		f_a=1.0-f_c
		absfile_qty=f_a*(*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty
		absfile_err=f_a*(*(*(*davePtr).dataStrPtr).commonStr.histPtr).err
		fa_tnorm=f_a*tnorm
		lines[icomp*2+4]="Processing absorber file "+filename
		dcs_message_display,lines,base,group_leader=dcs_base,alignment=0
		bases[icomp*2+1]=base
	endif
;
	if (filename ne absfile and filename ne canfile) then begin
		qty=(*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty
		err=(*(*(*davePtr).dataStrPtr).commonStr.histPtr).err
;
		scorr_qty=((qty - f_a*absfile_qty - f_c*canfile_qty) - $
			sumfb_qty*(tnorm - fa_tnorm - fc_tnorm))/sumeff_qty
		scorr_err=sqrt(((err^2 + (f_a*absfile_err)^2 + (f_c*canfile_err)^2) + $
			sumfb_err^2*(tnorm-fa_tnorm-fc_tnorm)^2))/sumeff_qty
		(*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty=scorr_qty
		(*(*(*davePtr).dataStrPtr).commonStr.histPtr).err=scorr_err
;
		strput,filename,"cor.dave",strlen(filename)-8
		grpfiles[i]=filename
		lines[icomp*2+4]="Writing sample output file "+filename
		dcs_message_display,lines,base,group_leader=dcs_base,alignment=0
		bases[icomp*2+1]=base
;
;	widget_control,/hourglass
;
		(*(*davePtr).dataStrPtr).commonStr.ylabel = 'mean Q'
		(*(*davePtr).dataStrPtr).commonStr.yunits = 'wavevector:A-1'
		(*(*davePtr).dataStrPtr).commonStr.ytype = 'points'
		(*(*davePtr).dataStrPtr).commonStr.histlabel = 'Grouped Intensity'
;
		(*(*(*davePtr).dataStrPtr).commonStr.treatmentPtr)=$
			strsplit(logentry,string(10B),/extract)
;
		save,davePtr,filename=filename
;
		temperatures[i]=(*(*(*davePtr).dataStrPtr).specificPtr).temp_setpoint > 0
;
	endif
;
	ptr_free,qtyPtr
	ptr_free,errPtr
	ptr_free,xPtr
endfor
;
; NB qflagPtr (if it exists) is a pointer array.
if (total(ptr_valid(qflagPtr)) gt 0) then ptr_free,qflagPtr
;
for icomp=2*nfiles-1,0,-1 do widget_control,bases[icomp],/destroy
;
free_all_ptrs,davePtr
;
;res=ptr_valid(count=nptr)
;print,'Exiting docorrections, ',nptr,' pointers in use.'
;
end