; $Id: fans_droplist.pro,v 1.4 2002/07/08 17:39:05 azuah Exp $
;===============================================================================
; Event Handler for FANS_DATREDUC program defined in fans_datreduc.pro.
; Deals with events triggered from the 3 droplist widgets which select
; or change the current data.
;
; Richard Tumanjong Azuah
;
; May 27 2002 - Initial writing
; 
;===============================================================================


;===============================================================================
; Updates the plot and slider widgets with the current data
pro updatePlotSlider, sPtr,dataType,dataPtr,err=err,emsg=emsg
; update plot
fans_plotData,sPtr,dataType

; update energy slider
fans_syncSlider,(*sPtr).eSliderID,(*dataPtr).energyPtr,0,err=err,emsg=emsg
if (err ne 0) then begin
    if (!debug) then printerror,err,emsg
    return
endif

end


;===============================================================================
pro fans_dropList, event

; Even when there is only one element in a droplist widget, on windows
; (but not linux), an event is still generated. This is a problem when
; the list _is_'empty' which in this case will contain an item 
; 'no files yet'. Deal with this by ignoring events when the number of
; items < 1.
nosItems = widget_info(event.id,/droplist_number)
if (nosItems le 1) then return

widget_control, event.top, get_uvalue=sPtr

dataType = widget_info(event.id,/uname)

switch dataType of
    
    'samp': begin

        ;; save selected index - useful when not in 'separate' mode
        (*sPtr).sampSepIndex = event.index

        ;; nothing to do if files are not analysed seperately
        if ((*(*sPtr).mfoPtr)[0] ne 2) then break

        ;; make selected data the current data
        (*sPtr).sampIndex = event.index
        
        ;; reset output calculation flag
        (*sPtr).doOutpCalc = 1

        ;; if the display option is 'Sample', then update plot and slider
        widget_control, (*sPtr).sViewID, get_uvalue=on
        if (on) then $
          updatePlotSlider, sPtr,dataType,(*(*sPtr).sDataPtr)[event.index],err=err,emsg=emsg

        ;; if the display option is 'Output', then update output plot
        widget_control, (*sPtr).oViewID, get_uvalue=on
        if (on) then begin    
            ;; Calculate output data
            fans_outpCalc, sPtr, err=err, emsg=emsg
            if (err ne 0) then begin
                printerror,err,emsg
                return
            endif
            fans_plotData, sPtr, 'outp', err=err,emsg=emsg
        endif

        break
    end


    'bkgd': begin
        ;; save selected index - useful when not in 'separate' mode
        (*sPtr).bkgdSepIndex = event.index

        ;; nothing to do if files are not analysed seperately
        if ((*(*sPtr).mfoPtr)[1] ne 2) then break

        ;; make selected data the current data
        (*sPtr).bkgdIndex = event.index

        ;; reset output calculation flag
        (*sPtr).doOutpCalc = 1

        ;; if the display option is 'Sample', then update plot
        widget_control, (*sPtr).bViewID, get_uvalue=on
        if (on) then $
          updatePlotSlider, sPtr,dataType,(*(*sPtr).bDataPtr)[event.index],err=err,emsg=emsg

        ;; if the display option is 'Output', then update output plot
        widget_control, (*sPtr).oViewID, get_uvalue=on
        if (on) then begin
            ;; Calculate output data
            fans_outpCalc, sPtr, err=err, emsg=emsg
            if (err ne 0) then begin
                printerror,err,emsg
                return
            endif
            fans_plotData, sPtr, 'outp', err=err,emsg=emsg
        endif

        break
    end

    'fast': begin

        ;; save selected index - useful when not in 'separate' mode
        (*sPtr).fastSepIndex = event.index

        ;; nothing to do if files are not analysed seperately
        if ((*(*sPtr).mfoPtr)[2] ne 2) then break

        ;; make selected data the current data
        (*sPtr).fastIndex = event.index

        ;; reset output calculation flag
        ;(*sPtr).doOutpCalc = 1

        ;; if the display option is 'Sample', then update plot
        widget_control, (*sPtr).fViewID, get_uvalue=on
        if (on) then $
          fans_plotData, sPtr, 'fast', err=err,emsg=emsg
        ;;updatePlotSlider, sPtr,dataType,(*(*sPtr).fDataPtr)[event.index],err=err,emsg=emsg

        break
    end


endswitch


end

