;$Id: rasda.pro,v 1.2 2002/04/25 21:35:34 amunter Exp $

pro roneasda,file_name,data,xtitle=xtitle,ytitle=ytitle
;------------------------------------------------------------------------
;|									|
;| Purpose	: Read ASD data-file of the format which is produced    |
;|		  by the NIST neutron spectrometer controle program 	|
;|		  ICP const-I buffer				    	|
;|									|
;| Input	: file-name, the data file			    	|
;| Output	: data, the data structure 				|
;| Externals	: gendast.pro					    	|
;| 									|
;| NOTE 	:							|
;| 	Angles of A4, 2thita, is assign to the horizontal axis	        |
;|	in this program							|
;------------------------------------------------------------------------

in			=	1000
junk			=	''
dfile			=	''
ddate			=	''
scanty			=	''
scantybase		=	''
npts			=	0
dtype			=	''
dtitle			=	''
dcollimation		=	intarr(4)
dmosaic			=	intarr(3)
ddet			=	intarr(1)

;------------------------------------------------
;| reading the headers			  	|
;------------------------------------------------

openr,lun,file_name,/get_lun
readf,lun,format='(x,a12,3x,a17,3x,a1,5x,f8.0,f7.0,1x,a4,2x,i4,3x,a3,2x)',$
      dfile,ddate,scanty,ddmon,prefact,scantybase,npts,dtype
dmon	=	ddmon*prefact
readf,lun,junk
readf,lun,format='(a100)',dtitle
readf,lun,dcollimation,dmosaic,dlamb,dtmpst,dtinc,dhfld,ddet
readf,lun,junk

;--------------------------------
;| start reading actual data 	|
;--------------------------------

;------------------------
;|  read angles		|
;------------------------
tt		=	fltarr(4,6)		;theta table
ttl		=	fltarr(4)
for i=0,5 do begin
	readf,lun,ttl
	tt(*,i)	=	ttl
endfor

readf,lun,junk
readf,lun,format='(x,a12,8x,i4)',junk,npix
readf,lun,junk

; asd data format
tempasd	=	fltarr(npix)
asd	=	fltarr(in,npix)

;Count the number of columns to be read. Assume standard set of columns.
iw	=	0
tiw	=	strarr(100)
while ( readword(junk,iw) ne '') do begin
	iw	=	iw+1
	tiw(iw-1)	=	readword(junk,iw-1)
end
tiw	=	tiw(0:iw-1)

;------------------------
;|  read intensities	|
;------------------------
itt	=	fltarr(in)
minin	=	fltarr(in)
tempar	=	fltarr(in)
tmp	=	fltarr(iw)
dtmp	=	fltarr(in,iw)
iu	=	0
on_ioerror, endwhil
while not eof(lun) do begin
	readf,lun,tmp
	dtmp(iu,*)	=	tmp
;print,reform(dtmp(iu,*))

	; asd data
	readf,lun,tempasd
	asd(iu,*)	=	tempasd

	iu		=	iu+1
	if iu ge 1000 then stop,'# of point is more than 1000' 
endwhil:  
endwhile
free_lun,lun

dtmp			=	dtmp(0:iu-1,*)
asd			=	asd(0:iu-1,*)

if iu ne npts then begin
	print,'(FROM rasda.pro) # of data points is different ' $
		+'from the # shown on top'
	print,'                  of the file. Is this an aborted file?'
endif
;------------------------------------------------
;| convert angle to real angle steps        	|
;------------------------------------------------
ary		=	fltarr(iu,6)		; arry of number of point


for la=0,5 do begin
	astp	=	tt(2,la)         	;angle step
	ma	=	tt(3,la)		;max angle
	mi	=	tt(1,la)		;min angle
	for lab=0,iu-1 do ary(lab,la)=mi+lab*astp
endfor


;________________________________________________
;|						|
;| start to generate the data structure		|
;|______________________________________________|
lis	=	where(tiw eq 'TEMP',ntem)
lis	=	where(tiw eq 'H-Field',nhf)

gendast,data,3,12,9+ntem,iu,npix,/asd
if ntem then begin
	if ntem then gendast,data,3,12,9+ntem+nhf,iu,npix,/asd	$
	else gendast,data,3,12,9+nhf,iu,npix,/asd	
end

data.tde(0)		=	'A4'
data.ude(0)		=	'degree'
data.tde(1)		=	'COUNTS'
data.ude(1)		=	'counts'
data.tde(2)		=	'error'
data.ude(2)		=	'counts'
data.tde(3)		=	'A2'
data.ude(3)		=	'degree'
data.tde(4)		=	'A3'
data.ude(4)		=	'degree'
data.tde(5)		=	'A5'
data.ude(5)		=	'degree'
data.tde(6)		=	'A6'
data.ude(6)		=	'degree'
data.tde(7)		=	'A1'
data.ude(7)		=	'degree'
data.tde(8)		=	'MIN'
data.ude(8)		=	'min.'
if ntem then begin
	data.tde(9)	=	'TEMP'
	data.ude(9)	=	'K'
endif	

if nhf then begin
	data.tde(n_elements(data.tde)-1)	=	'H-Field'
	data.ude(n_elements(data.tde)-1)	=	'Tesla'
endif	

for i=0,n_elements(tiw)-1 do begin
	data.de(*,where(data.tde eq tiw(i)))	=	dtmp(*,i)
end
data.de(*,2)		=	sqrt(data.de(*,1))
data.tde(1)		=	'Intensity'

; store angles
data.de(*,0)		=	ary(*,3)
data.de(*,3)		=	ary(*,1)
data.de(*,4)		=	ary(*,2)
data.de(*,5)		=	ary(*,4)
data.de(*,6)		=	ary(*,5)
data.de(*,7)		=	ary(*,0)

; store ASD data
data.asd(*,*)		=	asd(*,*)
data.dasd(*,*)		=	sqrt(asd(*,*))

;________________________________________________
;|						|
;|selecting the x,y axis			|
;|______________________________________________|
if n_elements(xtitle) eq 0 then sxya,data
saxis,data,xtitle=xtitle,ytitle=ytitle

data.inorm			=		0
inorm				=		data.inorm
data.nvh(inorm)			=		dmon
data.mon			=		dmon
if scantybase eq 'TIME' then begin
	data.tnvh(inorm)	=		'time'
	data.unvh(inorm)	=		'seconds'	
endif else begin
	data.tnvh(inorm)	=		'monitor'
	data.unvh(inorm)	=		'counts'	
endelse

data.tsvh(0)		=	'title'
data.tsvh(1)		=	'filename'
data.tsvh(2)		=	'date'
data.svh(0)		=	dtitle
data.svh(1)		=	dfile
data.svh(2)		=	ddate
data.scan		=	scanty

data.tnvh(1)		=	'wavelength'
data.unvh(1)		=	'AA^-1'
data.nvh(1)		=	dlamb
data.tnvh(2)		=	'H field'
data.unvh(2)		=	'Tesla'
data.nvh(2)		=	dhfld
data.tnvh(3)		=	'RM collim'
data.unvh(3)		=	'minutes of arc'
data.nvh(3)		=	dcollimation(0)
data.tnvh(4)		=	'MS collim'
data.unvh(4)		=	'minutes of arc'
data.nvh(4)		=	dcollimation(1)
data.tnvh(5)		=	'SA collim'
data.unvh(5)		=	'minutes of arc'
data.nvh(5)		=	dcollimation(2)
data.tnvh(6)		=	'AD collim'
data.unvh(6)		=	'minutes of arc'
data.nvh(6)		=	dcollimation(3)
data.tnvh(7)		=	'Ti'
data.unvh(7)		=	'K'
data.nvh(7)		=	dtmpst
data.tnvh(8)		=	'delt T'
data.unvh(8)		=	'K'
data.nvh(8)		=	dtinc
data.tnvh(9)		=	'mon mosaic'
data.unvh(9)		=	'minutes of arc'
data.nvh(9)		=	dmosaic(0)
data.tnvh(10)		=	'samp mosaic'
data.unvh(10)		=	'minutes of arc'
data.nvh(10)		=	dmosaic(1)
data.tnvh(11)		=	'ana mosaic'
data.unvh(11)		=	'minutes of arc'
data.nvh(11)		=	dmosaic(2)

	

lastline:
end



pro rasda,filename,data,xtitle=xtitle,ytitle=ytitle
;________________________________________________________________
;| Purpose	: read the data files into data structure	|
;|								|
;| Input	:						|
;|	-- filename, string array for filenames			|
;| 								|
;| External	:						|
;|	-- ronenista.pro	&	comb.pro  or subt.pro	|
;| 								|
;| 								|
;| Modifications :						|
;|	Name modified 2/13/93 by Collin Broholm also		|
;|	keyword changed to gassian not gaussan			|
;|______________________________________________________________|
common comwn,nwarno,nwartn
common comws,nwarts

nwartn		=	0
nwarts		=	0
nwarno		=	0
nfile		=	n_elements(filename)

	roneasda,filename(0),data,xtitle=xtitle,ytitle=ytitle
	strdu,data
if nfile le 1 then goto,onefile
for i=1,nfile-1 do begin
	roneasda,filename(i),datao,xtitle=xtitle,ytitle=ytitle
	strdu,datao
	merge,data,datao,d
	data	=	d
endfor


onefile:
end
